// Copyright 2022 Michal Przanowski <mike.przanowski@galacticgames.com>

#include "PlatformFunctionsBPLibrary.h"
#include "PlatformFunctions.h"
#include "AudioDevice.h"
#include "HAL/IConsoleManager.h"
#include "Misc/CommandLine.h"
#include "Engine/GameViewportClient.h"
#include "Framework/Application/SlateApplication.h"
#include "Framework/Application/SlateUser.h"

UPlatformFunctionsBPLibrary::UPlatformFunctionsBPLibrary(const FObjectInitializer& ObjectInitializer)
: Super(ObjectInitializer)
{

}

FString UPlatformFunctionsBPLibrary::GetCPUBrandName()
{
	return FPlatformMisc::GetCPUBrand();
}

FString UPlatformFunctionsBPLibrary::GetCPUVendorName()
{
	return FPlatformMisc::GetCPUVendor();
}

FString UPlatformFunctionsBPLibrary::GetGPUBrandName()
{
	return FPlatformMisc::GetPrimaryGPUBrand();
}

int32 UPlatformFunctionsBPLibrary::GetCPUCores()
{
	return FPlatformMisc::NumberOfCores();
}

FString UPlatformFunctionsBPLibrary::GetAudioDeviceName()
{
	FString DeviceName = "";
	TArray<FString> DeviceNames;

	if (GEngine)
	{
		TArray<FAudioDevice*> AudioDevices = GEngine->GetAudioDeviceManager()->GetAudioDevices();

		if (AudioDevices.Num() > 0)
		{
			AudioDevices[0]->GetAudioDeviceList(DeviceNames);

			if (DeviceNames.Num() > 0)
			{
				DeviceName = DeviceNames[0];
			}
		}
	}

	return DeviceName;
}

int32 UPlatformFunctionsBPLibrary::GetMonitorsCount()
{
	FDisplayMetrics Display;
	FDisplayMetrics::RebuildDisplayMetrics(Display);

	return Display.MonitorInfo.Num();
}

int32 UPlatformFunctionsBPLibrary::GetPrimaryMonitor()
{
	FDisplayMetrics Display;
	FDisplayMetrics::RebuildDisplayMetrics(Display);

	int32 MonitorNumber = 0;
	FParse::Value(FCommandLine::Get(), TEXT("monitor="), MonitorNumber);

	int32 MonitorIndex = INDEX_NONE;
	if (MonitorNumber == 0)
	{
		for (int32 Index = 0; Index < Display.MonitorInfo.Num(); Index++)
		{
			if (Display.MonitorInfo[Index].bIsPrimary)
			{
				MonitorIndex = Index;
				break;
			}
		}
	}
	else
	{
		MonitorIndex = MonitorNumber - 1;
	}

	return MonitorIndex;
}

bool UPlatformFunctionsBPLibrary::SetActiveMonitor(int32 MonitorIndex)
{
	if (MonitorIndex > GetMonitorsCount() - 1)
		return false;

	if (!GEngine)
		return false;

	if (!GEngine->GameViewport)
		return false;

	FDisplayMetrics Display;
	FDisplayMetrics::RebuildDisplayMetrics(Display);

	if (!Display.MonitorInfo.IsValidIndex(MonitorIndex))
		return false;

	const FMonitorInfo Monitor = Display.MonitorInfo[MonitorIndex];

	// (can be implemented in Blueprints)
	// Check if used monitor resolution is bigger then selected new monitor resolution if so change game resolution to selected monitor resolution
	// check if UserSettings.ResolutionSizeX > Monitor.NativeWidth || UserSettings.ResolutionSizeY > Monitor.NativeHeight
	// if true then change your game resolution to Monitor.NativeWidth, Monitor.NativeHeight

	const int32 WindowPosX = Monitor.WorkArea.Left;
	const int32 WindowPosY = Monitor.WorkArea.Top;
	const FVector2D Position(static_cast<float>(WindowPosX), static_cast<float>(WindowPosY));

	TSharedPtr<SWindow> Window = GEngine->GameViewport->GetWindow();

	// Best to set window position in WindowMode for Linux
#if PLATFORM_LINUX
	const EWindowMode::Type CurrentWindowMode = Window->GetWindowMode();
	Window->SetWindowMode(EWindowMode::Windowed);
#endif

	Window->MoveWindowTo(Position);

#if PLATFORM_LINUX
	// Set WindowMode back to mode before MoveWindow
	Window->SetWindowMode(CurrentWindowMode);
#endif

	return true;
}

void UPlatformFunctionsBPLibrary::SetGamma(float Gamma)
{
	GEngine->DisplayGamma = FMath::Clamp(Gamma, 1.0f, 5.0f);
}

float UPlatformFunctionsBPLibrary::GetGamma()
{
	return GEngine->GetDisplayGamma();
}

